package space

import (
	"context"

	"github.com/pkg/errors"
)

// @microgen grpc
// @protobuf git.perx.ru/perxis/perxis-go/proto/spaces
// @grpc-addr content.spaces.Spaces
type Spaces interface {
	// Create - создает пространство для размещения контента
	// В процессе создание пространства выполняются следующие шаги:
	// 1. Создается запись о пространстве
	// 2. Создается БД пространства
	// 3. Создается master окружение
	// 4. Создаются необходимые схема согласно указанной конфигурации
	// 5. Выполняется копирование данных согласно указанной конфигурации
	// Шаги могут выполняться в отложенном режиме, при этом пространство может быть недоступно для
	// выполнения любых операций. Попытка обращения к пространству будет сопровождаться ошибкой
	// ErrUnavailable

	Create(ctx context.Context, space *Space) (created *Space, err error)
	Get(ctx context.Context, spaceId string) (space *Space, err error)
	List(ctx context.Context, orgId string) (spaces []*Space, err error)
	Update(ctx context.Context, space *Space) (err error)
	UpdateConfig(ctx context.Context, spaceId string, config *Config) (err error)
	Delete(ctx context.Context, spaceId string) (err error)
}

func IsSpaceAvailable(ctx context.Context, spcs Spaces, spaceId string) error {
	spc, err := spcs.Get(ctx, spaceId)

	if err != nil {
		return errors.Wrap(err, "space not available")
	}

	if spc.State != StateReady {
		return errors.New("space not available")
	}

	return nil
}
